package ihm;

import java.awt.*;
import java.io.PrintWriter;
import java.lang.String;
import java.util.*;
import java.awt.image.BufferedImage;
import java.awt.geom.Rectangle2D;
import java.io.Serializable;
import java.awt.geom.Point2D;

import moteur.ET;

/**
 * La classe symboles affichables du rseau.
 */
public class Symbole {

   private int       x;
   private int       y;
   private int       w;
   private int       h;
   private boolean   online;
   private int       id;
   private int       type;
   private String    name = null;
   private String    ip = null;
   private String    net = null;
   private IHM       ihm;
   private Vector    alerts;


/**
 * Constructeur.
 * img: l'image qui le reprsente
 * x,y: le coin superieur gauche
 * w,h: largeur/hauteur
 */
   public Symbole (IHM interf, int symbID, int elementType, boolean isOnline, int posx, int posy, int width, int height) {
      online = isOnline;
   
      ihm = interf;
   
      id = symbID;
      
      type = elementType;
   
      x = posx;
      y = posy;
      w = width;
      h = height;
      
      alerts = new Vector();
   }
   
/**
 * Constructeur.
 * img: l'image qui le reprsente
 * x,y: le coin superieur gauche
 * Ce constructeur dtermine la largeur et la hauteur comme
 * tant celles de l'image si elle est valide, 0 sinon.
 */
   public Symbole (IHM interf, int symbID, int type, boolean online, int posx, int posy) {
      this (interf, symbID, type, online, posx, posy, 0, 0);
      
      Image img = ihm.getImage(type);
      if (img != null) {
         w = img.getWidth(null);
         h = img.getHeight(null);
      }
   }
   

/**
 * Dplace le symbole relativement  sa position courante.
 */
   void moveOf(int dX, int dY) {
      x += dX;
      y += dY;
      if (x < 0) x = 0;
      if (y < 0) y = 0;
   }

/**
 * Dplace le symbole vers une position absolue.
 */
   void moveTo(int posX, int posY) {
      x = posX;
      y = posY;
      if (x < 0) x = 0;
      if (y < 0) y = 0;
   }
   
/**
 * Retourne un rectangle de la dimension du symbole, et 
 * positionn  sa location.
 */
   Rectangle getBounds () {
      return new Rectangle (x, y, w, h);
   }
   
/**
 * Test si le symbole possde une intersection commune
 * avec le rectangle spcifi.
 * Utile pour tester si le curseur de souris est au dessus,
 * en spcifiant un carr de largeur/hauteur 1,1.
 * x,y: coordonnes du point suprieur gauche du rectangle.
 * w,h: largeur/hauteur du rectangle.
 */
   boolean intersects(int x, int y, int w, int h) {
        return getBounds().intersects (x, y, w, h);
    } // end intersects
    
/**
 * Renvoie le nom unique du symbole.
 */
   public String getName() {
      return ((name == null) || name.equals("") ? ET.getTypeName(type) + " " + id : name);
   }
    
/**
 * Renvoie l'ID du symbole.
 */
   public int getID () {
      return id;
   }

/**
 * Renvoie l'IP du symbole
 */ 
   public String getIp() {
      return ((ip == null) || ip.equals("") ? "?.?.?.?" : ip);
   }
 
   public String getNet () {
      return ((net == null) || net.equals("") ? "?.?.?.?" : net);
   }
 
   public boolean getOnline () {
      return online;
   }
 
   public Point2D getCenter() {
      return new Point2D.Double (x + w/2, y + h/2);
   }

/**
 * Ajoute une alerte  la liste des alertes en attente pour ce symbole.
 */
   public void addAlert (String msg) {
      alerts.addElement (msg);
   }
 
/**
 * Renvoie et supprime l'alerte la plus ancienne si il y en a une.
 */
   public String popAlert () {
      if (alerts.size() > 0) {
         return (String)alerts.remove(0);
      } else {
         return null;
      }
   }
   
   public Vector popAllAlerts () {
      Vector ret = (Vector)alerts.clone();
      alerts.clear();
      return ret;
   }
   
/**
 * Indique le nombre d'alertes en attente pour ce symbole.
 */
   public int getAlertPendingCount () {
      return alerts.size();
   }

   public void setOnline (boolean isOnline) {
      online = isOnline;
   }
   
   public void setType (int newType) {
      type = newType;
   }

   public void setName (String newName) {
      name = newName;
   }
   
   public void setIp (String newIp) {
      ip = newIp;
   }
   
   public void setNet (String newNet) {
      if ((net == null) || !newNet.equals(net)) {
         net = newNet;
      }
   }
   
/**
 * La symbole se dessine tout seul sur un objet de type Graphics2D
 * C'est le symbole qui sait dans quel tat il est et donc comment
 * il doit se dessiner.
 */
   void drawIt(Graphics2D g) {
      g.drawImage (ihm.getImage(type), x, y, w, h, null);
      if (!online) {
         drawOffline(g);
      }
      if (alerts.size() > 0) {
         drawAlert(g);
      }
      
      String sName = getName();
      String sIp = getIp();
      Rectangle2D nameBounds = g.getFont().getStringBounds (sName, g.getFontRenderContext());
      g.setColor (new Color (25, 200, 25));
      g.drawString (sName, (int)(x-(nameBounds.getWidth()-w)/2), y-3);
      Rectangle2D ipBounds = g.getFont().getStringBounds (sIp, g.getFontRenderContext());
      g.setColor (new Color (25, 25, 200));
      g.drawString (sIp, (int)(x-(ipBounds.getWidth()-w)/2), (int)(y+h+ipBounds.getHeight()+3));
   }
    
/**
 * Le symbole doit pouvoir dessiner son contour de selection,
 * c'est  dire des marques qui montre que ce symbole est slctionn.
 * Cette mthode doit tre appele  la fin de
 * toute l'opration d'affichage pour que la slction
 * apparaisse au dessus du reste.
 */
   void drawSelection(Graphics2D g) {
      g.setPaint (Color.black);
      g.draw (new Rectangle (x-2,y-2,4,4));
      g.draw (new Rectangle (x+w-2,y+h-2,4,4));    
   }

/**
 * Le symbole doit pouvoir indiquer qu'il y a une ou plusieurs alertes
 * en attente le concernant.
 * Cette mthode dessine donc un effet graphique visible d'alerte.
 * Cette mthode doit tre appele  la fin de
 * toute l'opration d'affichage pour que l'effet d'alerte
 * apparaisse au dessus du reste.
 */
   void drawAlert(Graphics2D g) {
      // Dessine un rectangle rouge autour, pour signaler une alerte.
      g.setPaint (Color.red);
      g.draw (new Rectangle (x, y, w, h));
      g.draw (new Rectangle (x+1, y+1, w-2, h-2));
   }
   
/**
 * Le symbole doit pouvoir indiquer qu'il est offline sur le rseau.
 * Cette mthode dessine donc un effet graphique visible.
 * Cette mthode doit tre appele  la fin de
 * toute l'opration d'affichage pour que l'effet offline
 * apparaisse au dessus du reste.
 */
   void drawOffline(Graphics2D g) {
      // Dessine une croix rouge pour le marquer comme non actif.
      g.setColor (Color.red);
      g.drawLine (x, y, x+w, y+h);
      g.drawLine (x+w, y, x, y+h);
   }

/**
 * Le toString normal, renvoie en fait le nom de l'objet.
 */
    public String toString() {
      return getName();
    }

} // end Symb
