package moteur;

import java.io.File;
import java.util.Hashtable;
import java.util.Vector;
import ihm.IHMMoteur;
import reseau.MibNode;
import ihm.SymbBounds;

/**
 * L'interface Moteur - IHM.
 * <p>
 * Cette inerface prsente les quelques mthodes que doit
 * implmenter la classe du moteur qui sera visible par l'IHM.
 * </p>
 * @author      M2SSI SNMP
 * @version     %I%, %G%
 * @since       1.0
 */
public interface MoteurIHM {
   
   /**
    * <p>Renvoie le numro d'accs d'un code MIB</p>
    * 
    * @param codeMib le code mib sous forme de chane de caractres
    * @return l'entier correspondant
    */
   public int getAccess(String codeMib);

   /**
    * <p> Renvoie le noeud de l'arbre de la MIB  partir du code</p>
    * 
    * @param codeMib le code mib sous forme de chane de caractres
    * @return le noeud de l'arbre MIB
    */
   public MibNode getNode(String codeMib);

   /**
    * <p>Renvoie la racine de l'arbre MIB</p>
    *
    * @return le noeud racine de l'arbre de la MIB
    */
   public MibNode getRoot();

   /**
   * <p> permet d'acceder au contenu du reseau </p>
   *
   * @return Reseau
   */

   public Reseau getReseau();

   /**
   * <p>permet d'acceder aux proprits des lments du rseau</p>
   *
   * @return ElementReseauConfiguration[]
   */
   public ElementReseauConfiguration [] getTabERC();


   /**
    * <p>Renvoie la valeur de la proptit online d'un lment rseau.</p>
    * <p>N'interroge pas le rseau.</p>
    *
    * @param id l'ID de l'lment rseau
    * @return true si l'lment est online, false sinon
    */
   public boolean getOnline(int id);

   /**
    * <p>Renvoie le type de l'lment rseau</p>
    * 
    * @param id l'identifiant de l'lment, partag entre le moteur et l'IHM
    * @return le type de l'lment. C'est l'un des types de la classe ET
    */
   public int getType(int id);

   /**
    * <p>Renvoie le nom de l'lment rseau</p>
    * 
    * @param id l'identifiant de l'lment, partag entre le moteur et l'IHM
    * @return le nom de l'lment
    */
   public String getName(int id);
   
   /**
    * <p>Renvoie l'IP de contact de l'lment rseau</p>
    * <p>L'IP de contact est ncessairement l'IP de l'une des interfaces accessibles de l'lment,
    * bien qu'il puisse y avoir plusieurs interface sur l'lment. C'est l'IP par lequel le moteur
    * contacte l'lment  travers la couche rseau.</p>
    * 
    * @param id l'identifiant de l'lment, partag entre le moteur et l'IHM
    * @return l'IP de contact de l'lment.
    */
   public String getIpContact(int id);

   /**
    * <p>Interroge le rseau pour vrifier l'tat de l'lment.</p>
    * <p>Le timeout est dfinit par le rseau, il est en gnral d'une seconde.
    * Cette fonction vrifie que l'lment rpond en SNMP, uniquement.</p>
    *
    * @param id l'ID de l'lment rseau
    * @return true si l'lment a t trouv, false sinon
    */
	public boolean isOnline(int id);

   /**
    * <p> permet de savoir si le moteur prend en charge les logs ou non </p>
    *
    * @return boolean
   */
   public boolean getLog();


    /**
     * <p>ouvre et initialise le moteur avec le fichier en paramtre</p>
     * <p>renvoie "true" si l'execution est correct, sinon renvoie le message d'erreur </p>
     *
     * @param fichier File
     * @return un tableau de donnes graphiques sur les lmnts.
     */
    public SymbBounds[] open(File fichier);


    /**
     * <p>sauvegarde le moteur dans le fichier en paramtre</p>
     * <p>renvoie "true" si l'execution est correct, sinon renvoie le message d'erreur </p>
     *
     * @param fichier File
     * @return String
     */
    public String save(File fichier, SymbBounds[] positions);



    /**
     * <p>sauvegarde le reseau dans le fichier en paramtre</p>
     * <p>renvoie "true" si l'execution est correct, sinon renvoie le message d'erreur </p>
     *
     * @param fichier File
     * @return String
     */
    public String export2Xml(File fichier);


    /**
     * <p> ajoute un lment au rseau</p>
     * <p> renvoie id si l'opration a russi, -1 sinon</p>
     *
     * @param type le type de l'lment  ajouter (tel que dfinit dans la classe ET)
     * @return l'ID de l'lment ajout. Cet ID sera partag par la suite entre l'IHM et le moteur
     */
    public int addElementReseau(int type);



    /**
     * <p> supprime un lment au rseau</p>
     * <p> renvoie true si l'opration a russi, false sinon</p>
     *
     * @param id l'ID de l'lment rseau
     * @return true si l'lment a t trouv, false sinon
     */
    public boolean removeElementReseau(int id);


    /**
     * <p>retourne les propriets de base a afficher pour l'element</p>
     *
     * @param id l'ID de l'lment rseau
     * @return une hashtable des proprits de base de l'lment
     */
    public Hashtable<String, String[]> getBase(int id);


    /**
     * <p>retourne les propriets personalises a afficher pour l'element</p>
     *
     * @param id l'ID de l'lment rseau
     * @return une hashtable des proprits personalises (favoris)
     */
    public Hashtable<String, String[]> getPerso(int id);



    /**
     *  <p>active la prise en charge des logs </p>
     */
    public void logOn();

    /**
     *  <p>dsactive la prise en charge des logs </p>
     */
    public void logOff();

    /**
     * <p>ajoute une proprit personalise a afficher</p>
     * <p> renvoie "true" si l'opration a russi, sinon un message d'erreur </p>
     *
     * @param id l'ID de l'lment rseau
     * @param key String
     * @param value String[]
     * @return String
     */
    public String addProprietePerso(int id, String key, String[] value);

    /**
     * <p>remplace une proprit personalis a afficher</p>
     * <p> renvoie "true" si l'opration a russi, sinon un message d'erreur </p>
     *
     * @param id l'ID de l'lment rseau
     * @param key String
     * @param old String[]
     * @param value String[]
     * @return String
     */
    public String replaceProprietePerso(int id, String key, String [] old, String[] value);

    /**
     * <p>supprime une proprit personalis a afficher</p>
     * <p> renvoie "true" si l'opration a russi, sinon un message d'erreur </p>
     *
     * @param id l'ID de l'lment rseau
     * @param key String
     * @return String
     */
    public String removeProprietePerso(int id, String key);

    /**
     * <p>ajoute une proprit de base a afficher</p>
     * <p> renvoie "true" si l'opration a russi, sinon un message d'erreur </p>
     *
     * @param id l'ID de l'lment rseau
     * @param key String
     * @param value String[]
     * @return String
     */
    public String addProprieteBase(int id, String key, String[] value);

    /**
     * <p>remplace une proprits de base a afficher</p>
     * <p> renvoie "true" si l'opration a russi, sinon un message d'erreur </p>
     *
     * @param id l'ID de l'lment rseau
     * @param key String
     * @param old String[]
     * @param value String[]
     * @return String
     */
    public String replaceProprieteBase(int id, String key, String [] old , String[] value);

    /**
     * <p>supprime une proprits de base a afficher</p>
     * <p> renvoie "true" si l'opration a russi, sinon un message d'erreur </p>
     *
     * @param id l'ID de l'lment rseau
     * @param key String
     * @param value String[]
     * @return String
     */
    public String removeProprieteBase(int id, String key, String [] value);

    /**
     * <p>supprime le reseau et ses proprits</p>
     * <p> renvoie "true" si l'opration a russi, sinon un message d'erreur </p>
     * @return String
     */
    public String delete();

    /**
     * <p>met  jour les proprits de base a afficher</p>
     * <p> renvoie "true" si l'opration a russi, sinon un message d'erreur </p>
     *
     * @param id l'ID de l'lment rseau
     * @param old Hashtable
     * @param value Hashtable
     * @return String
     */
    public String updateBase(int id, Hashtable<String, String[]> old, Hashtable<String, String[]> value);

    /**
     * <p>met  jour les proprits personalises a afficher</p>
     * <p> renvoie "true" si l'opration a russi, sinon un message d'erreur </p>
     *
     * @param id l'ID de l'lment rseau
     * @param old Hashtable
     * @param value Hashtable
     * @return String
     */
    public String updatePerso(int id, Hashtable<String, String[]> old, Hashtable<String, String[]> value);

    /**
     * Demande la fermeture de l'application.
     * <p>
     * Demande au moteur de fermer l'application proprement.
     * Le moteur doit utiliser les mthodes <code>finalize()</code> de l'IHM et
     * du rseau avant de terminer l'application.
     * </p>
     */
    public void endApp ();

    /**
     * Renvoie l'adresse de rseau de l'lment spcifi.
     * <p>
     * Le moteur calcul puis renvoie, sous forme de chane de caractres,
     * l'adresse du rseau auquel appartient l'lment spcifi dans l'argument.
     * </p>
     *
     * @param id l'ID de l'lment rseau
     * @return l'adresse du rseau, en tant que String.
     */
    public String getNetwork (int id);
}
